import logging
import traceback
from time import time
from reportportal_client import ReportPortalServiceAsync
from .variables import Variables


def async_error_handler(exc_info):
    exc, msg, tb = exc_info
    traceback.print_exception(exc, msg, tb)


def timestamp():
    return str(int(time() * 1000))


class RobotService(object):
    rp = None

    status_mapping = {
        "PASS": "PASSED",
        "FAIL": "FAILED",
        "SKIP": "SKIPPED"
    }

    log_level_mapping = {
        "INFO": "INFO",
        "FAIL": "ERROR",
        "TRACE": "TRACE",
        "DEBUG": "DEBUG"
    }

    @staticmethod
    def init_service(endpoint, project, uuid, log_batch_size):
        if RobotService.rp is None:
            logging.debug(
                "ReportPortal - Init service: "
                "endpoint={0}, project={1}, uuid={2}"
                .format(endpoint, project, uuid))
            RobotService.rp = ReportPortalServiceAsync(
                endpoint=endpoint,
                project=project,
                token=uuid,
                error_handler=async_error_handler,
                log_batch_size=log_batch_size)
        else:
            raise Exception("RobotFrameworkService is already initialized")

    @staticmethod
    def terminate_service():
        if RobotService.rp is not None:
            RobotService.rp.terminate()

    @staticmethod
    def start_launch(launch_name, mode=None, launch=None):
        sl_pt = {
            "name": launch_name,
            "start_time": timestamp(),
            "description": launch.doc,
            "mode": mode,
            "tags": Variables.launch_tags
        }
        logging.debug("ReportPortal - Start launch: "
                      "request_body={0}".format(sl_pt))
        RobotService.rp.start_launch(**sl_pt)

    @staticmethod
    def finish_launch(launch=None):
        fl_rq = {
            "end_time": timestamp(),
            "status": RobotService.status_mapping[launch.status]
        }
        logging.debug("ReportPortal - Finish launch: "
                      "request_body={0}".format(fl_rq))
        RobotService.rp.finish_launch(**fl_rq)

    @staticmethod
    def start_suite(name=None, suite=None):
        start_rq = {
            "name": name,
            "description": suite.doc,
            "tags": [],
            "start_time": timestamp(),
            "item_type": "SUITE"
        }
        logging.debug(
            "ReportPortal - Start suite: "
            "request_body={0}".format(start_rq))
        RobotService.rp.start_test_item(**start_rq)

    @staticmethod
    def finish_suite(issue=None, suite=None):
        fta_rq = {
            "end_time": timestamp(),
            "status": RobotService.status_mapping[suite.status],
            "issue": issue
        }
        logging.debug(
            "ReportPortal - Finish suite:"
            " request_body={0}".format(fta_rq))
        RobotService.rp.finish_test_item(**fta_rq)

    @staticmethod
    def start_test(test=None):
        start_rq = {
            "name": test.name,
            "description": test.doc,
            "tags": test.tags,
            "start_time": timestamp(),
            "item_type": "TEST"
        }
        logging.debug(
            "ReportPortal - Start test: "
            "request_body={0}".format(start_rq))
        RobotService.rp.start_test_item(**start_rq)

    @staticmethod
    def finish_test(issue=None, test=None):
        fta_rq = {
            "end_time": timestamp(),
            "status": RobotService.status_mapping[test.status],
            "issue": issue
        }
        logging.debug(
            "ReportPortal - Finish test:"
            " request_body={0}".format(fta_rq))
        RobotService.rp.finish_test_item(**fta_rq)

    @staticmethod
    def start_keyword(keyword=None):
        start_rq = {
            "name": keyword.get_name(),
            "description": keyword.doc,
            "tags": keyword.tags,
            "start_time": timestamp(),
            "item_type": keyword.get_type()
        }
        logging.debug(
            "ReportPortal - Start keyword: "
            "request_body={0}".format(start_rq))
        RobotService.rp.start_test_item(**start_rq)

    @staticmethod
    def finish_keyword(issue=None, keyword=None):
        fta_rq = {
            "end_time": timestamp(),
            "status": RobotService.status_mapping[keyword.status],
            "issue": issue
        }
        logging.debug(
            "ReportPortal - Finish keyword:"
            " request_body={0}".format(fta_rq))
        RobotService.rp.finish_test_item(**fta_rq)

    @staticmethod
    def log(message):
        sl_rq = {
            "time": timestamp(),
            "message": message.message,
            "level": RobotService.log_level_mapping[message.level],
            "attachment": message.attachment,
        }
        RobotService.rp.log(**sl_rq)



import logging

from .variables import Variables
from .model import Keyword, Test, Suite, LogMessage
from .service import RobotService

ROBOT_LISTENER_API_VERSION = 2


class ParentTypes(object):

    items = []

    @staticmethod
    def push(item):
        ParentTypes.items.append(item)

    @staticmethod
    def pop():
        return ParentTypes.items.pop()

    @staticmethod
    def peek():
        return ParentTypes.items[-1]

    @staticmethod
    def is_empty():
        return ParentTypes.items == []


def start_suite(name, attributes):
    suite = Suite(attributes=attributes)
    ParentTypes.push("SUITE")
    if attributes["id"] == "s1":
        Variables.check_variables()
        RobotService.init_service(Variables.endpoint, Variables.project,
                                  Variables.uuid, Variables.log_batch_size)
        suite.doc = Variables.launch_doc
        logging.debug("ReportPortal - Start Launch: {0}".format(attributes))
        RobotService.start_launch(launch_name=Variables.launch_name,
                                  launch=suite)
    else:
        logging.debug("ReportPortal - Start Suite: {0}".format(attributes))
        RobotService.start_suite(name=name, suite=suite)


def end_suite(name, attributes):
    suite = Suite(attributes=attributes)
    ParentTypes.pop()
    if attributes["id"] == "s1":
        logging.debug(msg="ReportPortal - End Launch: {0}".format(attributes))
        RobotService.finish_launch(launch=suite)
        RobotService.terminate_service()
    else:
        logging.debug("ReportPortal - End Suite: {0}".format(attributes))
        RobotService.finish_suite(suite=suite)


def start_test(name, attributes):
    test = Test(name=name, attributes=attributes)
    ParentTypes.push("TEST")
    logging.debug("ReportPortal - Start Test: {0}".format(attributes))
    RobotService.start_test(test=test)


def end_test(name, attributes):
    test = Test(name=name, attributes=attributes)
    ParentTypes.pop()
    logging.debug("ReportPortal - End Test: {0}".format(attributes))
    RobotService.finish_test(test=test)


def start_keyword(name, attributes):
    parent_type = "SUITE" if ParentTypes.is_empty() else ParentTypes.peek()
    kwd = Keyword(name=name, parent_type=parent_type, attributes=attributes)
    logging.debug("ReportPortal - Start Keyword: {0}".format(attributes))
    RobotService.start_keyword(keyword=kwd)


def end_keyword(name, attributes):
    kwd = Keyword(name=name, attributes=attributes)
    logging.debug("ReportPortal - End Keyword: {0}".format(attributes))
    RobotService.finish_keyword(keyword=kwd)


def log_message(message):
    # Check if message comes from our custom logger or not
    if isinstance(message["message"], LogMessage):
        msg = message["message"]
    else:
        msg = LogMessage(message["message"])
        msg.level = message["level"]

    logging.debug("ReportPortal - Log Message: {0}".format(message))
    RobotService.log(message=msg)


def message(message):
    logging.debug("ReportPortal - Message: {0}".format(message))


def library_import(name, attributes):
    logging.debug("ReportPortal - Library Import: {0}".format(attributes))


def resource_import(name, attributes):
    logging.debug("ReportPortal - Resource Import: {0}".format(attributes))


def variables_import(name, attributes):
    logging.debug("ReportPortal - Variables Import: {0}".format(attributes))


def output_file(path):
    logging.debug("ReportPortal - Output File: {0}".format(path))


def log_file(path):
    logging.debug("ReportPortal - Log File: {0}".format(path))


def report_file(path):
    logging.debug("ReportPortal - Report File: {0}".format(path))


def xunit_file(path):
    logging.debug("ReportPortal - XUnit File: {0}".format(path))


def debug_file(path):
    logging.debug("ReportPortal - Debug File: {0}".format(path))



from robot.libraries.BuiltIn import BuiltIn

from .exception import RobotServiceException


def get_variable(name, default=None):
    return BuiltIn().get_variable_value("${" + name + "}", default=default)


class Variables(object):
    uuid = None
    endpoint = None
    launch_name = None
    project = None
    launch_doc = None
    log_batch_size = None
    launch_tags = None

    @staticmethod
    def check_variables():
        Variables.uuid = get_variable("RP_UUID", default=None)
        if Variables.uuid is None:
            raise RobotServiceException(
                "Missing parameter RP_UUID for robot run\n"
                "You should pass -v RP_UUID:<uuid_value>")
        Variables.endpoint = get_variable("RP_ENDPOINT", default=None)
        if Variables.endpoint is None:
            raise RobotServiceException(
                "Missing parameter RP_ENDPOINT for robot run\n"
                "You should pass -v RP_RP_ENDPOINT:<endpoint_value>")
        Variables.launch_name = get_variable("RP_LAUNCH", default=None)
        if Variables.launch_name is None:
            raise RobotServiceException(
                "Missing parameter RP_LAUNCH for robot run\n"
                "You should pass -v RP_LAUNCH:<launch_name_value>")
        Variables.project = get_variable("RP_PROJECT", default=None)
        if Variables.project is None:
            raise RobotServiceException(
                "Missing parameter RP_PROJECT for robot run\n"
                "You should pass -v RP_PROJECT:<project_name_value>")
        Variables.launch_doc = get_variable("RP_LAUNCH_DOC", default=None)
        Variables.launch_tags = get_variable("RP_LAUNCH_TAGS", default="").split(" ")
        Variables.log_batch_size = get_variable("RP_LOG_BATCH_SIZE", default="20")



"""Report Portal logging API for test libraries with support for attachments.

Usage of this logger is similar to the standard robot.api.logger with addition
of an extra kwarg "attachment" to all logging functions.

Example
-------

import subprocess
from robotframework_reportportal import logger

class MyLibrary(object):

    def log_free_memory(self):
        logger.debug("Collecting free memory statistics!")
        logger.debug(
            "Memory consumption report",
            attachment={
                "name": "free_memory.txt",
                "data": subprocess.check_output("free -h".split()),
                "mime": "application/octet-stream",
            },
        )
"""

from robot.api import logger
from .model import LogMessage


def write(msg, level="INFO", html=False, attachment=None):
    """Writes the message to the log file using the given level.

    Valid log levels are ``TRACE``, ``DEBUG``, ``INFO`` (default since RF
    2.9.1), ``WARN``, and ``ERROR`` (new in RF 2.9). Additionally it is
    possible to use ``HTML`` pseudo log level that logs the message as HTML
    using the ``INFO`` level.

    Attachment should contain a dict with "name", "data" and "mime" values
    defined. See module example.

    Instead of using this method, it is generally better to use the level
    specific methods such as ``info`` and ``debug`` that have separate
    ``html`` argument to control the message format.
    """
    log_message = LogMessage(msg)
    log_message.level = level
    log_message.attachment = attachment
    logger.write(log_message, level, html)


def trace(msg, html=False, attachment=None):
    """Writes the message to the log file using the ``TRACE`` level."""
    write(msg, "TRACE", html, attachment)


def debug(msg, html=False, attachment=None):
    """Writes the message to the log file using the ``DEBUG`` level."""
    write(msg, "DEBUG", html, attachment)


def info(msg, html=False, also_console=False, attachment=None):
    """Writes the message to the log file using the ``INFO`` level.

    If ``also_console`` argument is set to ``True``, the message is
    written both to the log file and to the console.
    """
    write(msg, "INFO", html, attachment)
    if also_console:
        console(msg)


def warn(msg, html=False, attachment=None):
    """Writes the message to the log file using the ``WARN`` level."""
    write(msg, "WARN", html, attachment)


def error(msg, html=False, attachment=None):
    """Writes the message to the log file using the ``ERROR`` level."""
    write(msg, "ERROR", html, attachment)


def console(msg, newline=True, stream="stdout"):
    """Writes the message to the console.

    If the ``newline`` argument is ``True``, a newline character is
    automatically added to the message.

    By default the message is written to the standard output stream.
    Using the standard error stream is possibly by giving the ``stream``
    argument value ``'stderr'``.
    """
    logger.console(msg, newline, stream)



from setuptools import setup, find_packages

requirements = [
    "reportportal-client>=3.0.0"
]

setup(
    name='robotframework-reportportal',
    packages=find_packages(),
    version='3.0.0',
    description='Listener for RobotFramework reporting to ReportPortal',
    author='Artsiom Tkachou',
    author_email='SupportEPMC-TSTReportPortal@epam.com',
    url='https://github.com/reportportal/agent-Python-RobotFramework',
    download_url='https://github.com/reportportal/agent-Python-RobotFramework/tarball/3.0.0',
    keywords=['testing', 'reporting', 'robot framework', 'reportportal'],
    classifiers=[],
    install_requires=requirements
)
